#include "main.h"
#include "stm32f4xx_adc.h"
#include "stm32f4xx_dma.h"

#include "misc.h"

extern uint8_t currentTimeBase;
extern uint8_t couplingPos2;

volatile u8 ADCDMAfilled = 0;
extern volatile u16 st_TIM4_IRQHandler;

#define ADC_DMA                      DMA2         //  ADC Stream0(or 4) Channel_0
#define ADC_DMA_STREAM               DMA2_Stream0
#define ADC_DMA_CHANNEL              DMA_Channel_0
#define ADC_DMA_STREAM_IRQ           DMA2_Stream0_IRQn
#define ADC_DMA_STREAM_IRQHANDLER    DMA2_Stream0_IRQHandler

//110MHz        15(us)*55/11=75
//2   55/11          75=15us(1us)        188=37,5us(2,5us)  375=75us(5us)                              
//2   55/11        750=150us(10us)  1875=375us(25us)  3750=750us(50us)
//2   55/11      7500=1,5ms(0,1ms)    –-/4  27,5/11    3750=1,5ms(0,1ms)----
//2   55/23      8967=3,75ms(0,25ms)    –-/4  27,5/23    4484=3,75ms(0,25ms)---
//2   55/64      6445=7,5ms(0,5ms)    ---/6  18,3/36    3819=7,5ms(0,5ms)---
//2   55/92      8967=15ms(1ms)     ---/6  18,2/64    4297=15ms(1ms)---
//2 55/(144+8)   13569=37,5ms(2,5ms)     --//4 27,5/120   8594=37,5ms(2,5ms)    ---/6  18,3/152  4523=37,5ms(2,5ms)---
//2  55/488     8453=75ms(5ms)
//4 27,5/488   8453=150ms(10ms)

void setupADC(){
  ADC_CommonInitTypeDef ADC_CommonInitStructure;
  ADC_InitTypeDef ADC_InitStructure;

  ADC_DeInit();
  
  ADC_CommonInitStructure.ADC_Mode = ADC_Mode_Independent; 
  ADC_CommonInitStructure.ADC_Prescaler = ADC_Prescaler_Div2;
  ADC_CommonInitStructure.ADC_DMAAccessMode = ADC_DMAAccessMode_1;
  ADC_CommonInitStructure.ADC_TwoSamplingDelay = ADC_TwoSamplingDelay_5Cycles;
  
  // настройки ADC
  switch(currentTimeBase){
   //2   55/11       75=15us(1us)     188=37,5us(2,5us)  375=75us(5us)                              
   //2   55/11      750=150us(10us)  1875=375us(25us)   3750=750us(50us)
    case 12:  //1uS
    case 11:  //2.5uS
    case 10:  //5uS
    case 9:   //10uS
    case 8:   //25uS
    case 7:   //50uS
      //clock for ADC
//      ADC_CommonInitStructure.ADC_Prescaler = ADC_Prescaler_Div2;
      ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_3Cycles);
      ADC_RegularChannelConfig(ADC1, ADC_Channel_3, 2, ADC_SampleTime_3Cycles);
    break;
    case 6:  //100uS
     //2   55/11      7500=1,5ms(0,1ms)   –-/4  27,5/11    3750=1,5ms(0,1ms)----
//      ADC_CommonInitStructure.ADC_Prescaler = ADC_Prescaler_Div2;
      ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_3Cycles);
      ADC_RegularChannelConfig(ADC1, ADC_Channel_3, 2, ADC_SampleTime_3Cycles);
    break;
    case 5:  //250uS
     //2   55/23      8967=3,75ms(0,25ms)   –-/4  27,5/23    4484=3,75ms(0,25ms)---
//      ADC_CommonInitStructure.ADC_Prescaler = ADC_Prescaler_Div2;
      ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_15Cycles);
      ADC_RegularChannelConfig(ADC1, ADC_Channel_3, 2, ADC_SampleTime_15Cycles);
    break;
    case 4:   //500uS
     //2   55/64      6445=7,5ms(0,5ms)   ---/6  18,3/36    3819=7,5ms(0,5ms)---
//      ADC_CommonInitStructure.ADC_Prescaler = ADC_Prescaler_Div2;
      ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_56Cycles);
      ADC_RegularChannelConfig(ADC1, ADC_Channel_3, 2, ADC_SampleTime_56Cycles);
    break;
    case 3:   //1mS
     //2   55/92      8967=15ms(1ms)      ---/6  18,2/64    4297=15ms(1ms)---
//      ADC_CommonInitStructure.ADC_Prescaler = ADC_Prescaler_Div2;
      ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_84Cycles);
      ADC_RegularChannelConfig(ADC1, ADC_Channel_3, 2, ADC_SampleTime_84Cycles);
    break;
    case 2:  //2.5mS
     //2 55/(144+8)   13569=37,5ms(2,5ms)   ---/6  18,3/152  4523=37,5ms(2,5ms)---
//      ADC_CommonInitStructure.ADC_Prescaler = ADC_Prescaler_Div2;
      ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_144Cycles);
      ADC_RegularChannelConfig(ADC1, ADC_Channel_3, 2, ADC_SampleTime_144Cycles);
    break;
    case 1: //5mS
     //2  55/488     8453=75ms(5ms)
//      ADC_CommonInitStructure.ADC_Prescaler = ADC_Prescaler_Div2;
      ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_480Cycles);
      ADC_RegularChannelConfig(ADC1, ADC_Channel_3, 2, ADC_SampleTime_480Cycles);
    break;
    case 0: //10mS
     //4 27,5/488(480+8)   8452=150ms(10ms)
      ADC_CommonInitStructure.ADC_Prescaler = ADC_Prescaler_Div4;
      ADC_RegularChannelConfig(ADC1, ADC_Channel_1, 1, ADC_SampleTime_480Cycles);
      ADC_RegularChannelConfig(ADC1, ADC_Channel_3, 2, ADC_SampleTime_480Cycles);
    break;
  }
  ADC_CommonInit(&ADC_CommonInitStructure);
  ADC_StructInit(&ADC_InitStructure);

  ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
  ADC_InitStructure.ADC_Resolution = ADC_Resolution_8b;
  ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_T1_CC1;         
  ADC_InitStructure.ADC_ExternalTrigConvEdge = ADC_ExternalTrigConvEdge_None;
  ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
  if (couplingPos2==2)
   {
    ADC_InitStructure.ADC_ScanConvMode = DISABLE;
    ADC_InitStructure.ADC_NbrOfConversion = 1; //single channel
    DMA_SetCurrDataCounter(ADC_DMA_STREAM, NUM_SAMPLES*2);
   }
   else
   {
    ADC_InitStructure.ADC_ScanConvMode = ENABLE;
    ADC_InitStructure.ADC_NbrOfConversion = 2; //two channels
    DMA_SetCurrDataCounter(ADC_DMA_STREAM, NUM_SAMPLES);
   }
  ADC_Init(ADC1, &ADC_InitStructure);

  // Configures the ADC DMA
  ADC_DMARequestAfterLastTransferCmd(ADC1, ENABLE);
    
  // Enable the ADC DMA
  ADC_DMACmd(ADC1, ENABLE);

  // Enable ADC1
  ADC_Cmd(ADC1, ENABLE);

  ADC_SoftwareStartConv(ADC1);
}

void initADC(){
DMA_InitTypeDef DMA_InitStructure;
NVIC_InitTypeDef NVIC_InitStructure;

    DMA_Cmd(ADC_DMA_STREAM, DISABLE);
    while(DMA_GetCmdStatus(ADC_DMA_STREAM)==ENABLE);

    DMA_DeInit(ADC_DMA_STREAM);
    DMA_StructInit(&DMA_InitStructure);

    DMA_InitStructure.DMA_Channel = ADC_DMA_CHANNEL;
    DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)&ADC->CDR;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_MemoryDataSize_Byte;//DMA_PeripheralDataSize_HalfWord;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;//DMA_MemoryDataSize_HalfWord;
    DMA_InitStructure.DMA_BufferSize = NUM_SAMPLES;
    DMA_InitStructure.DMA_Memory0BaseAddr = (uint32_t)chCapture;
    DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralToMemory;
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
    DMA_InitStructure.DMA_Priority = DMA_Priority_High;
    DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Disable;
    DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_HalfFull;
    DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
    DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;

    DMA_Init(ADC_DMA_STREAM, &DMA_InitStructure);
    DMA_ClearITPendingBit(ADC_DMA_STREAM, DMA_IT_TCIF0);
    DMA_Cmd(ADC_DMA_STREAM, ENABLE);
     while(DMA_GetCmdStatus(ADC_DMA_STREAM)==DISABLE);

    DMA_ITConfig(ADC_DMA_STREAM, DMA_IT_TC, ENABLE);

    // enable DMA IRQ
    NVIC_InitStructure.NVIC_IRQChannel = ADC_DMA_STREAM_IRQ;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

//    SET_ADCOption1();  
    setupADC();
}

void startADCDMA(void){

  ADCDMAfilled = 2;
GPIO_ResetBits(GPIOC, GPIO_Pin_13);
  DMA_Cmd(ADC_DMA_STREAM, DISABLE);
  // wait for DMA-Stream disable
  while(DMA_GetCmdStatus(ADC_DMA_STREAM)==ENABLE);

  ADC_Cmd(ADC1, DISABLE);

  DMA_Cmd(ADC_DMA_STREAM, ENABLE);
  // wait for DMA-Stream enable
  while(DMA_GetCmdStatus(ADC_DMA_STREAM)==DISABLE);

  ADC_Cmd(ADC1, ENABLE);

  ADC_SoftwareStartConv(ADC1);
}

void ADC_DMA_STREAM_IRQHANDLER(void) //void DMA1_Channel1_IRQHandler(void)
{
  if (DMA_GetITStatus(ADC_DMA_STREAM, DMA_IT_TCIF0))
  {
    DMA_ClearITPendingBit(ADC_DMA_STREAM, DMA_IT_TCIF0);
    DMA_Cmd(ADC_DMA_STREAM, DISABLE);
    ADC_Cmd(ADC1, DISABLE);
    while(DMA_GetCmdStatus(ADC_DMA_STREAM)==ENABLE);
    if (ADCDMAfilled!=0) ADCDMAfilled--;
  GPIO_SetBits(GPIOC, GPIO_Pin_13);
  }
}
