// TFT display constants

#define TFT_WIDTH   320
#define TFT_HEIGHT    240
#define GRID_WIDTH    300
#define GRID_HEIGHT   200

#define ADC_MAX_VAL   256
#define ADC_GRID      256

#include "stm32f4xx.h"
#include "video.h"
#include "gdi.h"
#include "string.h"
#include <stdlib.h>
#include "main.h"

// global capture variables

unsigned char chCapture[NUM_SAMPLES*2] = {0};
// rendered waveform data is stored here for erasing
unsigned char ch1Old[GRID_WIDTH+1] = {0};
unsigned char ch2Old[GRID_WIDTH+1] = {0};
unsigned char sMess[7];
extern volatile u8 Key_mode_single, Key_mode, Key_plus, Key_minus;
u8 Key_mode_prev=0,Key_plus_prev=0, Key_minus_prev=0;
extern u8 Key_plus_cnt, Key_minus_cnt;
extern volatile u8 ADCDMAfilled;

uint32_t sIndex = 0;
int16_t trigLevel = 130;
bool triggerRising = 1;

uint8_t zeroVoltageA1S=129;
uint8_t zeroVoltageA2S=129;

volatile u8 hold_mode = RUN_m;
double freq1, freq2;
double Vavrf1, Vavrf2;

const char* cplNames[] = {" DC", " AC", "OFF"}; 
uint8_t couplingPos1=0;
uint8_t couplingPos2=0;

const char* rngNames[] = {"   4V", "   2V", "   1V", "500mV", "250mV"}; 
uint8_t rangePos1=2;
uint8_t rangePos2=2;

// this represents the drawing offset voltage of channels 1 and 2
volatile int16_t zeroOffsetVoltageA1=120, zeroOffsetVoltageA2=20;

// timebase enumerations and store
#define TBK    15*CPU_CLK/2 
//110MHz        15(us)*55/11=75
//u16 tbCnt[]={            8452,    8453,    13569,    8967,    6445,    8967,    7500,    3750,    1875,    750,     375,     188,     76};
u16 tbCnt[]={10000*TBK/(2*480), 5000*TBK/480, 2500*TBK/152, 1000*TBK/92, 500*TBK/64, 250*TBK/23, 100*TBK/11, 50*TBK/11, 25*TBK/11, 10*TBK/11, 5*TBK/11, 25*TBK/(11*10), TBK/11};
const char* tbNames[]={" 10mS", "  5mS",      "2.5mS",      "  1mS",     "500uS",    "250uS",    "100uS",    " 50uS",   " 25uS",   " 10uS",   "  5uS",  "2.5uS",        "  1uS"}; 
uint8_t currentTimeBase=8;

// out test frequency
const char* tfNames[] = {"T-OFF", " 250K", " 100K", "10KHz", " 5KHz", " 1KHz", "500Hz", "100Hz"}; 
uint8_t Mode=2, Mode_prev=1;
extern void SetTimerOut(u8 mode);
uint8_t Fout_mode=0;

// grid variables
uint8_t hOffset = (TFT_WIDTH - GRID_WIDTH)/2;
uint8_t vOffset = (TFT_HEIGHT - GRID_HEIGHT)/2;

// prints waveform statistics on screen
bool printStats = true;
// repaint the labels on screen in draw loop
bool paintLabels = true;//false;

const char* trgNames[] = {"TRIG1 ", "TRIG2 "}; 
uint8_t triggerType;

void drawStats(void);
void clearNDrawSignals(void);
void drawGrid(void);
void drawLabels(void);
void gdiLineV(uint16_t x1, uint16_t y1, uint16_t y2);
void calculateStats(u8 ch);
uint16_t FindMaxMin(uint32_t k1,uint32_t k2,uint16_t pr,uint8_t incr);

// ------------------------
void NumToStr(u32 Num)  {
sMess[0]=(Num%100000)/10000+0x30;
sMess[1]=(Num%10000)/1000+0x30;
sMess[2]=(Num%1000)/100+0x30;
sMess[3]=(Num%100)/10+0x30;
sMess[4]=(Num%10)+0x30;
sMess[5]=0;
}

// ------------------------
void CalcFreq(u32 Num)  {
if (Num<1000) //  999 Hz
{
 sMess[0]=sMess[1]=sMess[2]=sMess[6]=' ';
 sMess[3]=(Num%1000)/100+0x30;
 sMess[4]=(Num%100)/10+0x30;
 sMess[5]=(Num%10)+0x30;
}else if (Num<1000000) // 1.000KHz - 99.999KHz
{
 sMess[0]=(Num%100000)/10000+0x30;
 if (sMess[0]==0x30) sMess[0]=' ';
 sMess[1]=(Num%10000)/1000+0x30;
 sMess[2]='.';
 sMess[3]=(Num%1000)/100+0x30;
 sMess[4]=(Num%100)/10+0x30;
 sMess[5]=(Num%10)+0x30;
 sMess[6]='K';
}else if (Num<10000000) //100.00KHz - 999.99KHz
{
 sMess[0]=(Num%1000000)/100000+0x30;
 sMess[1]=(Num%100000)/10000+0x30;
 sMess[2]=(Num%10000)/1000+0x30;
 sMess[3]='.';
 sMess[4]=(Num%1000)/100+0x30;
 sMess[5]=(Num%100)/10+0x30;
 sMess[6]='K';
} else // 1.000MHz -  9.999MHz
{
 sMess[0]=' ';
 sMess[1]=(Num%10000000)/1000000+0x30;
 sMess[2]=(Num%1000000)/100000+0x30;
 sMess[3]='.';
 sMess[4]=(Num%100000)/10000+0x30;
 sMess[5]=(Num%10000)/1000+0x30;
 sMess[6]='M';
}
}

// ------------------------
void initDisplay(void)  {
// ------------------------
  vidClearScreen();
  gdiRectangle(0,0,(VID_PIXELS_X - 1),VID_VSIZE - 1,0);

  // and paint o-scope
    drawGrid();
    gdiRectangle(hOffset, vOffset, GRID_WIDTH+hOffset, GRID_HEIGHT+vOffset,0);
//    gdiRectangle(8,3,38,13,GDI_ROP_COPY); //RUN
    gdiRectangle(50,3,90,13,GDI_ROP_COPY); //TRIGx
}

// ------------------------
void drawWaves(void)  {
// ------------------------
  if ((hold_mode==RUN_m)||(hold_mode==SINGL_m)) clearNDrawSignals();
  
    drawStats();

  // if label repaint requested - do so now
  if(paintLabels) {
    drawLabels();
    paintLabels = false;
  }
}
// ------------------------
uint16_t FindMaxMin(uint32_t k1,uint32_t k2,uint16_t pr,uint8_t incr)
{
uint16_t max,min,val,val2;
  
  max = pr;
  min = pr;
while (k1 < k2) {
  val=chCapture[k1];
  if(max < val) max = val;
  if(min > val) min = val;
  k1+=incr;
 }
if (max>pr) val = max-pr; else val = pr-max;
if (min<pr) val2 = pr-min; else val2 = min-pr;
if (val<val2) max=min;
return max;
} 

// ------------------------
void clearNDrawSignals(void)  {
// ------------------------
static bool wavesOld[2] = {false,false};// this represents the offset voltage at ADC input (1.65V), when Analog input is zero
bool wavesSnap[2];
int16_t val1, val2;
int32_t i,j;
uint32_t k;
int32_t valt, kt, kts;
uint16_t prev,prev2;

  if (couplingPos1==2) wavesSnap[0]=0; else wavesSnap[0]=1;
  if (couplingPos2==2) wavesSnap[1]=0; else wavesSnap[1]=1;

if (couplingPos2==2) kt=1;  else kt=2;

  if (rangePos1==4)  for( k = 0; k < NUM_SAMPLES; k+=kt) {valt=(uint16_t)chCapture[k]*2-(uint16_t)zeroVoltageA1S;   if (valt>255) valt=255; chCapture[k]=valt;}
  if (rangePos2==4)  for( k = 1; k < NUM_SAMPLES; k+=2) {valt=(uint16_t)chCapture[k]*2-(uint16_t)zeroVoltageA2S;   if (valt>255) valt=255; chCapture[k]=valt;}

  kts = tbCnt[currentTimeBase]/kt;
  k = NUM_SAMPLES-kts/*tbCnt[currentTimeBase]*/*2;
  if ((triggerType/2)==0) {
    i = 20;
    j = 0;
    do{
     valt = (100+trigLevel-zeroOffsetVoltageA1)*ADC_GRID/GRID_HEIGHT;
     if(triggerType%2) 
      {if (((chCapture[i+kt]<=valt)&&(chCapture[i]>valt))||((chCapture[i+kt]<valt)&&(chCapture[i]>=valt))) j=i;}
       else
      {if (((chCapture[i]<=valt)&&(chCapture[i+kt]>valt))||((chCapture[i]<valt)&&(chCapture[i+kt]>=valt))) j=i;}
     i+=kt;
     } while( (j==0)&&(i < k) );
    } else {
    i = 21;
    j = 1;
    do{
     valt = (100+trigLevel-zeroOffsetVoltageA2)*ADC_GRID/GRID_HEIGHT;
     if(triggerType%2) 
      {if (((chCapture[i+2]<=valt)&&(chCapture[i]>valt))||((chCapture[i+2]<valt)&&(chCapture[i]>=valt))) j=i;}
       else
      {if (((chCapture[i]<=valt)&&(chCapture[i+2]>valt))||((chCapture[i]<valt)&&(chCapture[i+2]>=valt))) j=i;}
     i+=2;
     } while( (j==1)&&(i < (k+1)) );
    }   
    if ((triggerType/2)!=0) j--;
    
    sIndex=j;

    if ((hold_mode==SINGL_m)&&(j>1)) {hold_mode=STOP_m;paintLabels = true;}

//   NumToStr(Mode);
//   NumToStr(ADCDMAfilled);
//   NumToStr(zeroVoltageA1S);
//   gdiDrawTextEx(200, 4, (unsigned char *)sMess, GDI_ROP_COPY);
//   NumToStr(chCapture[sIndex]);
//   NumToStr(zeroVoltageA2S);
//   gdiDrawTextEx(235, 4, (unsigned char *)sMess, GDI_ROP_COPY);

  // go through all the data points
  for( i = 0; i < GRID_WIDTH; i++)  {
    // erase old line segment ch2
    if(wavesOld[1]) {
      val1 = ch2Old[i];
      val2 = ch2Old[i+1];
      if (i!=(GRID_WIDTH - 1)) gdiLine(NULL,hOffset + i + 1,val1, hOffset + i + 1, val2, GDI_ROP_XOR);
    }
    // erase old line segment ch1
    if(wavesOld[0]) {
      val1 = ch1Old[i];
      val2 = ch1Old[i+1];
      if (i!=(GRID_WIDTH - 1)) 
        {gdiLine(NULL,hOffset + i + 1,val1, hOffset + i + 1, val2, GDI_ROP_COPY);
         gdiLine(NULL,hOffset + i + 1,val1, hOffset + i + 1, val2, GDI_ROP_XOR);}
    }
    // draw new segments ch1
    if(wavesSnap[0])  {
      if (kts>299)
      {
        if (i==0) prev = chCapture[j];
        valt=prev;
        val1 = ((valt - zeroVoltageA1S) * GRID_HEIGHT)/ADC_GRID;
        if (couplingPos2==2)
          prev=FindMaxMin(   ((int32_t)kts*i)/300 +j,  ((int32_t)kts*(i+1))/300+j,prev,1);//1 CH
         else
          prev=FindMaxMin(2*(((int32_t)kts*i)/300)+j,2*((int32_t)kts*(i+1))/300+j,prev,2);//2 CH
        valt=prev;
        val2 = ((valt - zeroVoltageA1S) * GRID_HEIGHT)/ADC_GRID;
      }
       else
      {
        k = (i*kts)/300;
        kt = i*10-(3000*k)/kts;
        if (couplingPos2==2)
         {
          k= k+j;
          valt = ((chCapture[k+1] - chCapture[k])*kt)*kts/3000 + chCapture[k];
         }
         else
         {
          k= k*2+j;
          valt = ((chCapture[k+2] - chCapture[k])*kt)*kts/3000 + chCapture[k];
         }
        val1 = ((valt - zeroVoltageA1S) * GRID_HEIGHT)/ADC_GRID;

        k = (i+1)*kts/300;
        kt = (i+1)*10-(3000*k)/kts;
        if (couplingPos2==2)
         {
          k= k+j;
          valt = ((chCapture[k+1] - chCapture[k])*kt)*kts/3000 + chCapture[k];
         }
         else
         {
          k= k*2+j;
          valt = ((chCapture[k+2] - chCapture[k])*kt)*kts/3000 + chCapture[k];
         }
        val2 = ((valt - zeroVoltageA1S) * GRID_HEIGHT)/ADC_GRID;
      }
       
      val1 =  220-val1-zeroOffsetVoltageA1;
      if (val1<(vOffset+1)) val1=vOffset+1;
      if (val1>(GRID_HEIGHT+vOffset-1)) val1=(GRID_HEIGHT+vOffset-1);

      val2 = 220-val2-zeroOffsetVoltageA1;
      if (val2<(vOffset+1)) val2=vOffset+1;
      if (val2>(GRID_HEIGHT+vOffset-1)) val2=(GRID_HEIGHT+vOffset-1);

      ch1Old[i] = val1;
      if (i!=(GRID_WIDTH - 1)) gdiLine(NULL,hOffset + i + 1, val1, hOffset + i + 1, val2, GDI_ROP_COPY);
    }
    // draw new segments ch2
    if(wavesSnap[1])  {
      if (kts>299)
      {
        if (i==0) prev2 = chCapture[j+1];
        valt=prev2;
        val1 = ((valt - zeroVoltageA2S) * GRID_HEIGHT)/ADC_GRID;
        prev2=FindMaxMin(2*(((int32_t)kts*i)/300)+j+1,(2*((int32_t)kts*(i+1))/300)+j+1,prev2,2);
        valt=prev2;
        val2 = ((valt - zeroVoltageA2S) * GRID_HEIGHT)/ADC_GRID;
      }
       else
      {
        k = (i*kts)/300;
        kt = i*10-(3000*k)/kts;
        k= k*2+j+1;
        valt = ((chCapture[k+2] - chCapture[k])*kt)*kts/3000 + chCapture[k];
        val1 = ((valt - zeroVoltageA2S) * GRID_HEIGHT)/ADC_GRID;

        k = (i+1)*kts/300;
        kt = (i+1)*10-(3000*k)/kts;
        k= k*2+j+1;
        valt = ((chCapture[k+2] - chCapture[k])*kt)*kts/3000 + chCapture[k];
        val2 = ((valt - zeroVoltageA2S) * GRID_HEIGHT)/ADC_GRID;
      }

      val1 =  220-val1-zeroOffsetVoltageA2;
      if (val1<(vOffset+1)) val1=vOffset+1;
      if (val1>(GRID_HEIGHT+vOffset-1)) val1=(GRID_HEIGHT+vOffset-1);

      val2 = 220-val2-zeroOffsetVoltageA2;
      if (val2<(vOffset+1)) val2=vOffset+1;
      if (val2>(GRID_HEIGHT+vOffset-1)) val2=(GRID_HEIGHT+vOffset-1);
       
      ch2Old[i] = val1;
      if (i!=(GRID_WIDTH - 1)) gdiLine(NULL,hOffset + i + 1, val1, hOffset + i + 1, val2, GDI_ROP_COPY);
    }
  }

  // store the drawn parameters to old storage
  wavesOld[0] = wavesSnap[0];
  wavesOld[1] = wavesSnap[1];
  drawGrid();
}

void gdiLineV(uint16_t x1, uint16_t y1, uint16_t y2) {
 gdiLine(NULL, x1, y1, x1 ,y1+y2, GDI_ROP_COPY);
}

void gdiLineH(uint16_t x1, uint16_t y1, uint16_t x2) {
 gdiLine(NULL, x1, y1, x2+x1 ,y1, GDI_ROP_COPY);
}

// ------------------------
void drawGrid(void) {
  uint8_t hPacing = GRID_WIDTH / 15;
  uint8_t vPacing = GRID_HEIGHT / 10;
  int i,j;
  
  for( i = 1; i < 15; i++)
  for( j = vOffset; j <(GRID_HEIGHT + vOffset); j+=5)
      gdiPoint(NULL, i * hPacing + hOffset, j, GDI_ROP_COPY);

  for( i = 1; i < 10; i++)
  for( j = hOffset; j <(GRID_WIDTH + hOffset); j+=5)
      gdiPoint(NULL, j , i * vPacing + vOffset, GDI_ROP_COPY);
}

void DrawMode(u8 mode)
{
    switch(mode)
  {
    case 0:
    gdiRectangle(8,3,38,13,GDI_ROP_XOR); //RUN
    break;
    case 1:
    gdiRectangle(50,3,90,13,GDI_ROP_XOR); //TRIGx
    break;
    case 2:
    gdiRectangle(98,3,134,13,GDI_ROP_XOR); //25mS
    break;
    case 3:
    gdiRectangle(150,3,184,13,GDI_ROP_XOR); //T-OFF
    break;
    case 4:
    gdiRectangle(10,221,159,238,GDI_ROP_XOR); //<0> 1
    break;
    case 5:
    gdiRectangle(10,223,30,233,GDI_ROP_XOR); //DC
    break;
    case 6:
    gdiRectangle(32,223,64,233,GDI_ROP_XOR); //Range1
    break;
    case 7:
    gdiRectangle(162,221,310,238,GDI_ROP_XOR); //<0> 2
    break;
    case 8:
    gdiRectangle(162,223,182,233,GDI_ROP_XOR); //DC
    break;
    case 9:
    gdiRectangle(184,223,216,233,GDI_ROP_XOR); //Range2
    break;
    case 10:
    gdiRectangle(1,3,9,237,GDI_ROP_XOR);  //TRIG_L
    break;
  }
}

// ------------------------
void drawLabels() {
// ------------------------
int cPos, trigX;

  // paint run/hold information
  trigX = 12;
  switch(hold_mode)
  {
  case RUN_m:
   gdiDrawTextEx(trigX, 4, " RUN", GDI_ROP_COPY);
   break;
  case HOLD_m:
   gdiDrawTextEx(trigX, 4, "HOLD", GDI_ROP_COPY);
   break;
  case SINGL_m:
   gdiDrawTextEx(trigX, 4, "SINL", GDI_ROP_COPY);
   break;
  case STOP_m:
   gdiDrawTextEx(trigX, 4, "STOP", GDI_ROP_COPY);
   break;
  }
  // draw trigger level on left side
   gdiDrawTextEx(3,210 - trigLevel, " ", GDI_ROP_COPY);
   gdiDrawTextEx(3,222 - trigLevel, " ", GDI_ROP_COPY);
   gdiDrawTextEx(3,216 - trigLevel, "|", GDI_ROP_COPY);

  // draw zero level on right side
		
   gdiDrawTextEx(312,210 - zeroOffsetVoltageA2, " ", GDI_ROP_COPY);
   gdiDrawTextEx(312,222 - zeroOffsetVoltageA2, " ", GDI_ROP_COPY);
	 if (couplingPos2!=2)
    gdiDrawTextEx(312,216 - zeroOffsetVoltageA2, "*", GDI_ROP_COPY);
	   else gdiDrawTextEx(312,216 - zeroOffsetVoltageA2, " ", GDI_ROP_COPY);
  // draw zero level on right side
   gdiDrawTextEx(312,210 - zeroOffsetVoltageA1, " ", GDI_ROP_COPY);
   gdiDrawTextEx(312,222 - zeroOffsetVoltageA1, " ", GDI_ROP_COPY);
   gdiDrawTextEx(312,216 - zeroOffsetVoltageA1, "*", GDI_ROP_COPY);

  // draw trigger
  trigX += 40;
  gdiDrawTextEx(trigX, 4, (pu8)trgNames[triggerType/2], GDI_ROP_COPY);
  // draw trigger edge
  trigX += 30;
  cPos=12;
  if(triggerType%2) { gdiLineH(trigX + 2, cPos, 2); gdiLineV(trigX + 2, cPos -8, 8); gdiLineH(trigX, cPos - 8, 2); }
   else  { gdiLineH(trigX, cPos, 2); gdiLineV(trigX + 2, cPos - 8, 8); gdiLineH(trigX + 2, cPos - 8, 2); } 
  
  // print new timebase
  trigX += 20;
  gdiDrawTextEx(trigX, 4, (pu8)tbNames[currentTimeBase], GDI_ROP_COPY);

  trigX += 50;
  gdiDrawTextEx(trigX, 4, (pu8)tfNames[Fout_mode], GDI_ROP_COPY);

  // -----------------
  trigX = hOffset + 2;
  cPos=GRID_HEIGHT + vOffset + 4;
  gdiDrawTextEx(trigX, cPos, (pu8)cplNames[couplingPos1], GDI_ROP_COPY);
  trigX += 22;
  gdiDrawTextEx(trigX, cPos, (pu8)rngNames[rangePos1], GDI_ROP_COPY);

  gdiLineV(160, 221, 19);  
// -----------------
  trigX = hOffset + 154;
  gdiDrawTextEx(trigX, cPos, (pu8)cplNames[couplingPos2], GDI_ROP_COPY);
  trigX += 22;
  if (couplingPos2!=2)
   gdiDrawTextEx(trigX, cPos, (pu8)rngNames[rangePos2], GDI_ROP_COPY);
    else gdiDrawTextEx(trigX, cPos, "     ", GDI_ROP_COPY);
  if (Mode!=Mode_prev)
  {
    DrawMode(Mode_prev);
    Mode_prev=Mode;
    DrawMode(Mode);
  }
}

// ------------------------
void drawStats(void)  {
// ------------------------
//  static long lastCalcTime = 0;
//  bool clearStats = false;

  calculateStats(0); //ch1
  calculateStats(1); //ch2

   NumToStr(freq1/100);
   if (sMess[0]==0x30) sMess[0]=' ';
   gdiDrawTextEx(104, 224, (unsigned char *)sMess, GDI_ROP_COPY);
   NumToStr(freq1);
   gdiDrawTextEx(116, 224, (unsigned char *)sMess, GDI_ROP_COPY);
   gdiDrawTextEx(146, 224, "Hz", GDI_ROP_COPY);

	 if (couplingPos2!=2)
	 {
    NumToStr(freq2/100);
    if (sMess[0]==0x30) sMess[0]=' ';
    gdiDrawTextEx(256, 224, (unsigned char *)sMess, GDI_ROP_COPY);
    NumToStr(freq2);
    gdiDrawTextEx(268, 224, (unsigned char *)sMess, GDI_ROP_COPY);
    gdiDrawTextEx(298, 224, "Hz", GDI_ROP_COPY);
	 } else gdiDrawTextEx(256, 224, "         ", GDI_ROP_COPY);
   if (Vavrf1<0) {Vavrf1*=-1; NumToStr(Vavrf1); sMess[0]='-';} else {NumToStr(Vavrf1); sMess[0]=' ';}
   if (sMess[1]==0x30) {sMess[1]=sMess[2]; sMess[2]='.'; } else { sMess[4]=sMess[3]; sMess[3]='.';}
   gdiDrawTextEx(66, 224, (unsigned char *)sMess, GDI_ROP_COPY);
   gdiDrawTextEx(96, 224, "V", GDI_ROP_COPY);

	 if (couplingPos2!=2)
	 {
    if (Vavrf2<0) {Vavrf2*=-1; NumToStr(Vavrf2); sMess[0]='-';} else {NumToStr(Vavrf2); sMess[0]=' ';}
    if (sMess[1]==0x30) {sMess[1]=sMess[2]; sMess[2]='.'; } else { sMess[4]=sMess[3]; sMess[3]='.';}
    gdiDrawTextEx(218, 224, (unsigned char *)sMess, GDI_ROP_COPY);
    gdiDrawTextEx(248, 224, "V", GDI_ROP_COPY);
	 } else gdiDrawTextEx(218, 224, "        ", GDI_ROP_COPY);
}

// ------------------------
void calculateStats(u8 ch) {
// ------------------------
  // extract waveform stats
  int32_t Vmax = -ADC_MAX_VAL, Vmin = ADC_MAX_VAL;
  int64_t sumSamples = 0;
  int64_t sumSquares = 0;
  int64_t freqSumSamples = 0;
  uint32_t k;
  uint32_t fVavr;
  bool dnWave;
  bool firstOne = true;
  uint32_t cHigh = 0;
  uint32_t sCtr;
  int32_t val;
  
  uint32_t sumCW = 0;
  uint32_t sumPW = 0;
  uint32_t numCycles = 0;
  uint32_t numHCycles = 0;
  double tPerSample;
  double avgCycleWidth;
  
  for(k = 0; k < NUM_SAMPLES/2; k++) {
   if (ch==0)
    val = chCapture[k*2+ch]-zeroVoltageA1S/*ADC_GRID/2*/;
     else
    val = chCapture[k*2+ch]-zeroVoltageA2S/*ADC_GRID/2*/;
    if(Vmax < val)
      Vmax = val;
    if(Vmin > val)
      Vmin = val;

    sumSamples += val;
    freqSumSamples += chCapture[k*2+ch];
    sumSquares += (val * val);
  }
if (ch==0)
{
 if (couplingPos1==0)
   Vavrf1 = 2*sumSamples/NUM_SAMPLES;
  else
   Vavrf1 = Vmax - Vmin;
 switch(rangePos1)
 {
    case 0: //   4v
      Vavrf1*=(double)4000/255;
      break;
    case 1: //   2v
      Vavrf1*=(double)2000/255;
      break;
    case 2: //   1v
      Vavrf1*=(double)1000/255;
      break;
    case 3: //   0.5v
      Vavrf1*=(double)500/255;
      break;
    case 4: //   0.25v
      Vavrf1*=(double)250/255;
      break;
//    case 5: //   0.1v
//      Vavrf1*=(float)100/255;
//      break;
 }
} else {
 if (couplingPos2==0)
   Vavrf2 = 2*sumSamples/NUM_SAMPLES;
  else
   Vavrf2 = Vmax - Vmin;
 switch(rangePos2)
 {
    case 0: //   4v
      Vavrf2*=(double)4000/255;
      break;
    case 1: //   2v
      Vavrf2*=(double)2000/255;
      break;
    case 2: //   1v
      Vavrf2*=(double)1000/255;
      break;
    case 3: //   0.5v
      Vavrf2*=(double)500/255;
      break;
    case 4: //   0.25v
      Vavrf2*=(double)250/255;
      break;
//    case 5: //   0.1v
//      Vavrf2*=(float)100/255;
//      break;
 }
}

  // find out frequency
  fVavr = 2*freqSumSamples/NUM_SAMPLES;
  //if (fVavr==128) fVavr = (Vmax-Vmin)/2+ADC_GRID/2;
  dnWave  = (chCapture[sIndex+ch] < fVavr - 2);
//if (ch==0){
//   NumToStr(fVavr);
//   gdiDrawTextEx(200, 4, (unsigned char *)sMess, GDI_ROP_COPY);
//   NumToStr(dnWave);
//   gdiDrawTextEx(235, 4, (unsigned char *)sMess, GDI_ROP_COPY);
//   }
// trigger stopped at sIndex
  for(sCtr = 0, k = sIndex; sCtr < NUM_SAMPLES/2; sCtr++, k+=2) {
    if(k == NUM_SAMPLES)
      k = 0;
    // mark the points where wave transitions the average value
    if(dnWave && (chCapture[k+ch] > fVavr + 2))  {
      if(!firstOne) {
        sumCW += (sCtr - cHigh);
        numCycles++;
      }
      else
        firstOne = false;
      dnWave = false;
      cHigh = sCtr;
    }

    if(!dnWave && (chCapture[k+ch] < fVavr - 2)) {
      if(!firstOne) {
        sumPW += (sCtr - cHigh);
        numHCycles++;
      }
      dnWave = true;
    }
  }
  
  tPerSample = 0;
  switch(currentTimeBase)
  {
    case 12: //1uS
    case 11: //2.5uS
    case 10: //5uS
    case 9:  //10uS
    case 8:  //25uS
    case 7:  //50uS
      tPerSample = (double)11*2/CPU_CLK;
      break;
    case 6:  //100uS
      tPerSample = (double)11*2/CPU_CLK;
      break;
    case 5:  //250uS
      tPerSample = (double)23*2/CPU_CLK;
      break;
    case 4:   //500uS
      tPerSample = (double)64*2/CPU_CLK;
      break;
    case 3:   //1mS
      tPerSample = (double)92*2/CPU_CLK;
      break;
    case 2:  //2.5mS
      tPerSample = (double)152*2/CPU_CLK;
      break;
    case 1: //5mS
      tPerSample = (double)488*2/CPU_CLK;
      break;
    case 0: //10mS
      tPerSample = (double)488*4/CPU_CLK;
      break;
  }
  tPerSample*=2;
  avgCycleWidth = (double)sumCW * tPerSample / numCycles;
  
  if (ch==0)      
    freq1=(double)1000000/avgCycleWidth;
   else
    freq2=(double)1000000/avgCycleWidth;
  
}
/*
// ------------------------
void drawVoltage(float volt, int y, boolean mvRange)  {
// ------------------------
  // text is standard 5 px wide
  int numDigits = 1;
  int lVolt = volt;
  
  // is there a negative sign at front
  if(volt < 0)  {
    numDigits++;
    lVolt = -lVolt;
  }
  
  // how many digits before 0
  if(lVolt > 999)
    numDigits++;
  if(lVolt > 99)
    numDigits++;
  if(lVolt > 9)
    numDigits++;
  
  // mv range has mV appended at back
  if(mvRange) {
    numDigits += 1;
    int x = GRID_WIDTH + hOffset - 10 - numDigits * 5;
    tft.setCursor(x, y);
    int iVolt = volt;
//    tft.print(iVolt);
//    tft.print("m");
  }
  else  {
    // non mV range has two decimal pos and V appended at back
    numDigits += 3;
    int x = GRID_WIDTH + hOffset -10 - numDigits * 5;
//    tft.setCursor(x, y);
//    tft.print(volt);
  }

}
*/
void Delay(volatile uint32_t nCount) {
  for (; nCount != 0; nCount--);
}

void VdivCh2(u8 mode)
{
  switch(mode)
  {
    case 0: //   4v
      GPIO_SetBits(GPIOB, GPIO_Pin_0); // =1
      GPIO_SetBits(GPIOB, GPIO_Pin_1); // =1
      break;
    case 1: //   2v
      GPIO_ResetBits(GPIOB, GPIO_Pin_0); // =0
      GPIO_ResetBits(GPIOB, GPIO_Pin_1); // =0
      break;
    case 2: //   1v
      GPIO_SetBits(GPIOB, GPIO_Pin_0); // =1
      GPIO_ResetBits(GPIOB, GPIO_Pin_1); // =0
      break;
    case 3: //   0.4v
    case 4: //   0.2v
    case 5: //   0.1v
      GPIO_ResetBits(GPIOB, GPIO_Pin_0); // =0
      GPIO_SetBits(GPIOB, GPIO_Pin_1); // =1
      break;
  }
}
void VdivCh1(u8 mode)
{
  switch(mode)
  {
    case 0: //   4v
      GPIO_SetBits(GPIOC, GPIO_Pin_14); // =1
      GPIO_SetBits(GPIOC, GPIO_Pin_15); // =1
      break;
    case 1: //   2v
      GPIO_ResetBits(GPIOC, GPIO_Pin_14); // =0
      GPIO_ResetBits(GPIOC, GPIO_Pin_15); // =0
      break;
    case 2: //   1v
      GPIO_SetBits(GPIOC, GPIO_Pin_14); // =1
      GPIO_ResetBits(GPIOC, GPIO_Pin_15); // =0
      break;
    case 3: //   0.4v
    case 4: //   0.2v
    case 5: //   0.1v
      GPIO_ResetBits(GPIOC, GPIO_Pin_14); // =0
      GPIO_SetBits(GPIOC, GPIO_Pin_15); // =1
      break;
  }
}

void controlLoop()  {
// ------------------------
      if (Key_mode_single==2)
      {
       switch(Mode)
        {
        case 0:
        case 1:
        case 2:
        case 3:
          Mode=6; //rangePos1
          break;
        case 4:
        case 5:
        case 6:
          if (couplingPos2!=2) Mode=9; //rangePos2
				   else Mode=8; //couplingPos2
          break;
        case 7:
        case 8:
        case 9:
          Mode=10; //trigLevel
          break;
        case 10:
          Mode=2; //currentTimeBase
          break;
        }
        paintLabels = true;
        Key_mode_single=3;
      }
    if (Key_plus&(!Key_plus_prev))
    {
      Key_plus_prev++;
      if (Key_mode==1)
      {
         if (Mode!=10) 
				 {
					Mode++;
					if (couplingPos2==2) { if ((Mode==7)||(Mode==9)) Mode++; }
				 } else Mode=0;
         paintLabels = true;
      } else
      switch(Mode)
      {
        case 0:
          if (hold_mode<SINGL_m) hold_mode++; else hold_mode=SINGL_m;
          break;
        case 1:
					if (couplingPos2!=2) {if (triggerType!=3) triggerType++;}	else {if (triggerType!=1) triggerType++;}
          break;
        case 2:
          if (currentTimeBase!=12) currentTimeBase++;
          initADC();
          break;
        case 3:
          if (Fout_mode!=7) {Fout_mode++; SetTimerOut(Fout_mode);}
          break;
        case 4:
          if (zeroOffsetVoltageA1>0) {zeroOffsetVoltageA1-=2; Key_plus_cnt=0; Key_plus=0; Key_plus_prev=0;} else zeroOffsetVoltageA1=0;
          break;
        case 5:
          if (couplingPos1!=2) couplingPos1++;
          if (couplingPos1==0) {
             GPIO_SetBits(GPIOB, GPIO_Pin_9); 
            } else {
             GPIO_ResetBits(GPIOB, GPIO_Pin_9);
            }
          break;
        case 6:
          if (rangePos1!=4) rangePos1++;
          VdivCh1(rangePos1);
          break;
        case 7:
          if (zeroOffsetVoltageA2>0) {zeroOffsetVoltageA2-=2; Key_plus_cnt=0; Key_plus=0; Key_plus_prev=0;} else zeroOffsetVoltageA2=0;
          break;
        case 8:
          if (couplingPos2!=2) couplingPos2++;
          if (couplingPos2==0) {
             GPIO_SetBits(GPIOB, GPIO_Pin_2); 
            } else {
             GPIO_ResetBits(GPIOB, GPIO_Pin_2);
            }
					if (couplingPos2==2) {initADC(); if (triggerType>1) triggerType-=2;} // if CH2 disabled test and change TRIG2 to TRIG1
          break;
        case 9:
          if (rangePos2!=4) rangePos2++;
          VdivCh2(rangePos2);
          break;
        case 10:
          if (trigLevel>0) {trigLevel-=2; Key_plus_cnt=0; Key_plus=0; Key_plus_prev=0;} else trigLevel=0;
          break;
      }
        paintLabels = true;
    } else if ((!Key_plus)&Key_plus_prev){  Key_plus_prev--; Delay(200); }

    if (Key_minus&(!Key_minus_prev))
    {
      Key_minus_prev++;
      if (Key_mode==1)
      {
         if (Mode!=0)
				 {
				  Mode--;
					if (couplingPos2==2) { if ((Mode==7)||(Mode==9)) Mode--; }
				 } else Mode=10;
         paintLabels = true;
      } else
      switch(Mode)
      {
        case 0:
          if (hold_mode!=RUN_m) {if (hold_mode==STOP_m) hold_mode=HOLD_m; else hold_mode--;}
          break;
        case 1:
          if (triggerType!=0) triggerType--;
          break;
        case 2:
          if (currentTimeBase!=0) currentTimeBase--;
          initADC();
          break;
        case 3:
          if (Fout_mode!=0) {Fout_mode--; SetTimerOut(Fout_mode);}
          break;
        case 4:
          if (zeroOffsetVoltageA1<200) {zeroOffsetVoltageA1+=2; Key_minus_cnt=0; Key_minus=0; Key_minus_prev=0;} else zeroOffsetVoltageA1=200;
          break;
        case 5:
          if (couplingPos1!=0) couplingPos1--;
          if (couplingPos1==0) {
            GPIO_SetBits(GPIOB, GPIO_Pin_9); 
           } else {
            GPIO_ResetBits(GPIOB, GPIO_Pin_9);
           }
          break;
        case 6:
          if (rangePos1!=0) rangePos1--;
          VdivCh1(rangePos1);
          break;
        case 7:
          if (zeroOffsetVoltageA2<200) {zeroOffsetVoltageA2+=2; Key_minus_cnt=0; Key_minus=0; Key_minus_prev=0;} else zeroOffsetVoltageA2=200;
          break;
        case 8:
          if (couplingPos2!=0) couplingPos2--;
          if (couplingPos2==1) initADC();
          if (couplingPos2==0) {
            GPIO_SetBits(GPIOB, GPIO_Pin_2); 
           } else {
            GPIO_ResetBits(GPIOB, GPIO_Pin_2);
           }
          break;
        case 9:
          if (rangePos2!=0) rangePos2--;
          VdivCh2(rangePos2);
          break;
        case 10:
          if (trigLevel<200) {trigLevel+=2; Key_minus_cnt=0; Key_minus=0; Key_minus_prev=0;}
          break;
      }
      paintLabels = true; 
    } else  if ((!Key_minus)&Key_minus_prev){ Key_minus_prev--; Delay(200); }
}

// ------------------------
