/***************************************************************************
 * STM32 VGA demo
 * Copyright (C) 2012 Artekit Italy
 * http://www.artekit.eu
 * Written by Ruben H. Meleca
 
### main.c
 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#include "stm32f4xx.h"                  // Device header
#include "stm32f4xx_pwr.h"
#include "stm32f4xx_adc.h"
#include "stm32f4xx_flash.h"

#include "video.h"
#include "gdi.h"
#include "main.h"

#define CONST_FLASH_ADDR 0x08020000   //Sector 5

extern uint8_t Mode, Fout_mode;
extern volatile u16 st_TIM4_IRQHandler;
extern uint8_t zeroVoltageA1S;
extern uint8_t zeroVoltageA2S;
extern uint8_t couplingPos1;
extern uint8_t couplingPos2;
extern uint8_t rangePos1;
extern uint8_t rangePos2;
extern uint8_t chCapture[NUM_SAMPLES*2];
//DataBuffer dataBuffer;
extern volatile u8 ADCDMAfilled;
extern bool paintLabels;
volatile u8    DMABufferReady = 0; // Сигнал конца подготовки буфера
extern volatile u8 hold_mode;
extern volatile u16 vline;
extern void controlLoop(void);

void RCC_Configuration(void)
{
  RCC_DeInit();
  RCC_LSEConfig(RCC_LSE_OFF);
  RCC_HSEConfig(RCC_HSE_ON);
//  RCC_PLLConfig(RCC_PLLSource_HSE, 25 /*PLLM*/, 220 /*PLLN*/, 2 /*PLLP*/, 4 /*PLLQ*/); // 110MHz
  RCC_PLLConfig(RCC_PLLSource_HSE, 25 /*PLLM*/, CPU_CLK*2 /*PLLN*/, 2 /*PLLP*/, 4 /*PLLQ*/); // 100MHz
  RCC_PLLCmd(ENABLE);
  RCC_HCLKConfig(RCC_SYSCLK_Div1);
  RCC_PCLK1Config(RCC_HCLK_Div2);
  RCC_PCLK2Config(RCC_HCLK_Div1);
  RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
  PWR_MainRegulatorModeConfig(PWR_Regulator_Voltage_Scale1);

  /* TIM1,2,3,4, ADC1, SPI1, DMA, GPIOA, GPIOB, GPIOC clocks enable */
  RCC_AHB1PeriphClockCmd( RCC_APB1Periph_PWR | RCC_AHB1Periph_DMA2 | RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOB | RCC_AHB1Periph_GPIOC, ENABLE);
  RCC_APB1PeriphClockCmd( RCC_APB1Periph_TIM2 | RCC_APB1Periph_TIM3 | RCC_APB1Periph_TIM4, ENABLE);
  RCC_APB2PeriphClockCmd( RCC_APB2Periph_ADC1 | RCC_APB2Periph_SPI1, ENABLE);
}

void GPIO_Configuration(void)
{
/* Initialize LED which connected to PC13 */
    GPIO_InitTypeDef  GPIO_InitStructure;
// Col_KEY
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
//#define AN_CH1      PA1   // analog channel 2
//#define AN_CH4      PA3   // analog channel 4
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_3;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AN;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
//#define Fout   PB3 - TIM2_CH1:
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_PinAFConfig(GPIOB, GPIO_PinSource3, GPIO_AF_TIM2);
    GPIO_Init(GPIOB, &GPIO_InitStructure);
// key_shift  key_plus  key_minus
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12|GPIO_Pin_13|GPIO_Pin_14;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
// key_A AC/DC  A1  B1  key_B AC/DC -PB9
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0|GPIO_Pin_1|GPIO_Pin_2|GPIO_Pin_9;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
// test_LED  A2  B2
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13|GPIO_Pin_14|GPIO_Pin_15;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
    GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_Init(GPIOC, &GPIO_InitStructure);
}

void SetTimerOut(u8 mode)
{
TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
TIM_OCInitTypeDef TIM_OCInitStructure;

  TIM_Cmd(TIM2, DISABLE);
  
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  switch(mode)
  {
    case 0:
      TIM_TimeBaseStructure.TIM_Period = 0; //T-OFF
      break;
    case 1:
      TIM_TimeBaseStructure.TIM_Period = CPU_CLK*10000/(2500*2); //250Khz
      break;
    case 2:
      TIM_TimeBaseStructure.TIM_Period = CPU_CLK*10000/(1000*2); //100Khz
      break;
    case 3:
      TIM_TimeBaseStructure.TIM_Period = CPU_CLK*10000/(100*2);  //10Khz
      break;
    case 4:
      TIM_TimeBaseStructure.TIM_Period = CPU_CLK*10000/(50*2);  //5Khz
      break;
    case 5:
      TIM_TimeBaseStructure.TIM_Period = CPU_CLK*10000/(10*2);  //1Khz
      break;
    case 6:
      TIM_TimeBaseStructure.TIM_Period = CPU_CLK*10000/(5*2);   //500Hz
      break;
    case 7:
      TIM_TimeBaseStructure.TIM_Period = CPU_CLK*10000/(1*2);  //100Hz
      break;
  }
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
  TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Toggle;//TIM_OCMode_PWM2;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 1;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;
  TIM_OCInitStructure.TIM_OCNPolarity = TIM_OCNPolarity_High;
  if (mode)
  {
   TIM_OC2Init(TIM2, &TIM_OCInitStructure);
   TIM_CtrlPWMOutputs(TIM2, ENABLE);
   TIM_Cmd(TIM2, ENABLE);
  }
}
int main(void)
{
  uint32_t i;
  RCC_Configuration();
  GPIO_Configuration();
  vidInit();

  GPIO_SetBits(GPIOB, GPIO_Pin_2);  // LED_A AC/DC=1
  GPIO_SetBits(GPIOB, GPIO_Pin_0); // A1=1
  GPIO_ResetBits(GPIOB, GPIO_Pin_1); // B1=0
  
  GPIO_SetBits(GPIOB, GPIO_Pin_9); // LED_B AC/DC=1
  GPIO_SetBits(GPIOC, GPIO_Pin_14); // A2=1
  GPIO_ResetBits(GPIOC, GPIO_Pin_15); // B2=0

  initADC();
  initDisplay();
  SetTimerOut(0);

	i= *(uint32_t *)CONST_FLASH_ADDR;

	zeroVoltageA1S=i&0x0ff;
  zeroVoltageA2S=(i&0x0ff0000)>>16;

  while(1)
  {
    while(vline);
    controlLoop(); 
    if(paintLabels) {drawLabels(); paintLabels = false;}

    if(st_TIM4_IRQHandler%10==0)
    {
      if ((hold_mode==RUN_m)||(hold_mode==SINGL_m))
      {
       if (ADCDMAfilled==0) 
			 {	
        while(vline);
				if ((couplingPos1==1)&(couplingPos2==1)&(rangePos1==0)&(rangePos2==0)) 
         if (!GPIO_ReadInputDataBit(GPIOA,GPIO_Pin_0)) 
					{
						zeroVoltageA1S=chCapture[10];
						zeroVoltageA2S=chCapture[11];
						FLASH_Unlock();
						FLASH_EraseSector(FLASH_Sector_5,VoltageRange_3);
						i = zeroVoltageA1S;	i+=zeroVoltageA2S<<16;
						FLASH_ProgramWord(CONST_FLASH_ADDR,i);
						FLASH_Lock();
					}
				drawWaves();
			  startADCDMA();
			 }
      }
    }
  }
}
