/***************************************************************************
 * STM32 VGA demo
 * Copyright (C) 2012 Artekit Italy
 * http://www.artekit.eu
 * Written by Ruben H. Meleca
 
### video.c
 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/
#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_tim.h"
#include "stm32f4xx_spi.h"
#include "stm32f4xx_dma.h"
#include "misc.h"
#include "video.h"
#include "main.h"

#define VIDEO_DMA                DMA2         //  SPI1_TX Stream3 Channel_3
#define DMA_STREAM               DMA2_Stream3
#define DMA_CHANNEL              DMA_Channel_3
#define DMA_STREAM_IRQ           DMA2_Stream3_IRQn
#define DMA_STREAM_IRQHANDLER    DMA2_Stream3_IRQHandler

#define VTOTAL  42              /* Total bytes to send through SPI */
__align(4) u8 fb[VID_VSIZE][VID_HSIZE+2]; /* Frame buffer */
volatile u16 vline = 0;        /* The current line being drawn */
static volatile u32 vflag = 0;        /* When 1, the SPI DMA request can draw on the screen */
static volatile u32 vdraw = 0;        /* Used to increment vline every 3 drawn lines */ 
volatile u16 st_TIM4_IRQHandler;
volatile u8 Key_mode_single=0, Key_mode=0, Key_plus=0, Key_minus=0;
u8 Key_mode_cnt=0,Key_plus_cnt=0, Key_minus_cnt=0;

void TIMER_Configuration(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  NVIC_InitTypeDef nvic;
  TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
  TIM_OCInitTypeDef TIM_OCInitStructure;
  u32 TimerPeriod = 0;
  u16  Channel1PulseT3 = 0, Channel3PulseT3 = 0, Channel1PulseT4 = 0, Channel3PulseT4 = 0;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_6; //PORT_B4 - T3C1 <- PORT_B5 - T3C2, PORT_B6 - T4C1
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource4, GPIO_AF_TIM3);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_TIM4);
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  
  /*
    VGA 640x480 @ 60 Hz
    Vertical refresh  31.469 kHz
    Pixel freq.     24.0 MHz (12*8=96)/4
    Pixel freq.     27.0 MHz (12*9=108)/4

    1 system tick @ 96Mhz = 0,01041(6) us
    1 system tick @ 108Mhz = 0,009(259) us
  */

  /*
    Horizontal timing
    -----------------
    
    Timer 3 period = 31476 Hz
    
    Timer 3 channel 1 generates a pulse for HSYNC each 31.77 us.
    31.77 us = Visible area + Front porch + Sync pulse + Back porch.
    HSYNC is 3.77 us long, so the math to do is:
    3.77us / 0,010416us = 362 system ticks.   //HS
    3.77us / 0,009259us = 407 system ticks.   //HS

    Timer 3 channel 3 generates a pulse equal to HSYNC + back porch.
    This interrupt will fire the DMA request to draw on the screen if vflag == 1.
    Since firing the DMA takes more or less 800ns, we'll add some extra time.
    The math for HSYNC + back porch is:
    (3.77us + 1,89us - dma) / 0,010416us = +-466 system ticks   //HSBP
    (3.77us + 1,89us - dma) / 0,009259us = +-525 system ticks   //HSBP
  
    Horizontal timing info
    ----------------------
                  Dots  us
    --------------------------------------------    
    Visible area  640   25.17
    Front porch   8+8   0.94
    Sync pulse    96    3.77
    Back porch    40+8  1.89
    Whole line    800   31.77  //WL
  
  */

// VGA 640x480 @ 60 Hz    Pixel freq.     24.0 MHz (12*8=96)/4
//#define WL  3050+100   // 96000000/31476
//#define HS  362
//#define HSBP  466+25

// VGA 640x480 @ 60 Hz    Pixel freq.     27.0 MHz (12*9=108)/4
#define WL  CPU_CLK*1000000/31476 //(3430-200)   //  108000000/31476
#define HS  407
#define HSBP  (525+30)

  TimerPeriod = WL;
  Channel1PulseT3 = HS;   /* HSYNC */
  Channel3PulseT3 = HSBP; /* HSYNC + BACK PORCH */
  
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseStructure.TIM_Period = TimerPeriod;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
  TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);

  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Enable;
  TIM_OCInitStructure.TIM_Pulse = Channel1PulseT3;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;
  TIM_OCInitStructure.TIM_OCNPolarity = TIM_OCNPolarity_High;
  TIM_OCInitStructure.TIM_OCIdleState = TIM_OCIdleState_Reset;
  TIM_OCInitStructure.TIM_OCNIdleState = TIM_OCIdleState_Set;

  TIM_OC1Init(TIM3, &TIM_OCInitStructure);
  
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Inactive;
  TIM_OCInitStructure.TIM_Pulse = Channel3PulseT3;
  TIM_OC3Init(TIM3, &TIM_OCInitStructure);

  /* TIM3 counter enable and output enable */
  TIM_CtrlPWMOutputs(TIM3, ENABLE);

  /* Select TIM3 as Master */
  TIM_SelectMasterSlaveMode(TIM3, TIM_MasterSlaveMode_Enable);
  TIM_SelectOutputTrigger(TIM3, TIM_TRGOSource_Update);
  
  /*
    Vertical timing  =====  VGA 640x480 @ 60 Hz
    ---------------
    Polarity of vertical sync pulse is positive.
            Lines
    ------------------------------
    Visible area  480
    Front porch   2+8
    Sync pulse    2    // SPV
    Back porch    25+8 // SPBPV
    Whole frame   525  // WFV
  */

// VGA 640x480 @ 60 Hz
#define WFV  525
#define SPV  2
#define SPBPV  35

  TIM_SelectSlaveMode(TIM4, TIM_SlaveMode_Gated);
  TIM_SelectInputTrigger(TIM4, TIM_TS_ITR2);
  
  TimerPeriod = WFV;    /* Vertical lines */
  Channel1PulseT4 = SPV;    /* Sync pulse */
  Channel3PulseT4 = SPBPV;  /* Sync pulse + Back porch */
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseStructure.TIM_Period = TimerPeriod;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;

  TIM_TimeBaseInit(TIM4, &TIM_TimeBaseStructure);

  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Enable;
  TIM_OCInitStructure.TIM_Pulse = Channel1PulseT4;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;
  TIM_OCInitStructure.TIM_OCNPolarity = TIM_OCNPolarity_High;
  TIM_OCInitStructure.TIM_OCIdleState = TIM_OCIdleState_Reset;
  TIM_OCInitStructure.TIM_OCNIdleState = TIM_OCIdleState_Set;
  TIM_OC1Init(TIM4, &TIM_OCInitStructure);
  
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Inactive;
  TIM_OCInitStructure.TIM_Pulse = Channel3PulseT4;
  TIM_OC3Init(TIM4, &TIM_OCInitStructure);

  /*  TIM4 counter enable and output enable */
  TIM_CtrlPWMOutputs(TIM4, ENABLE);

  /* Interrupt TIM2 */
    nvic.NVIC_IRQChannel = TIM4_IRQn;
    nvic.NVIC_IRQChannelPreemptionPriority = 1;
    nvic.NVIC_IRQChannelSubPriority = 0;
    nvic.NVIC_IRQChannelCmd = ENABLE;

  NVIC_Init(&nvic);
  TIM_ITConfig(TIM4, TIM_IT_CC3, ENABLE);

  /* Interrupt TIM3 */
    nvic.NVIC_IRQChannel = TIM3_IRQn;
    nvic.NVIC_IRQChannelPreemptionPriority = 1;
    nvic.NVIC_IRQChannelSubPriority = 0;
    nvic.NVIC_IRQChannelCmd = ENABLE;

  NVIC_Init(&nvic);
  TIM_ITConfig(TIM3, TIM_IT_CC3, ENABLE);
  
  TIM_Cmd(TIM4, ENABLE);
  TIM_Cmd(TIM3, ENABLE);
}

void SPI_Configuration(void)
{
  NVIC_InitTypeDef nvic;
  SPI_InitTypeDef SPI_InitStructure;
  DMA_InitTypeDef DMA_InitStructure;
  GPIO_InitTypeDef GPIO_InitStructure;
  
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_5;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource5, GPIO_AF_SPI1);
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  SPI_I2S_DeInit(SPI1);
  SPI_Cmd(SPI1, DISABLE);
  DMA_DeInit(DMA_STREAM);

  DMA_StructInit(&DMA_InitStructure);
  DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)&SPI1->DR;
  DMA_InitStructure.DMA_Memory0BaseAddr = (u32) &fb[0][0];
  DMA_InitStructure.DMA_DIR = DMA_DIR_MemoryToPeripheral;
  DMA_InitStructure.DMA_BufferSize = VTOTAL;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Word; //DMA_MemoryDataSize_Byte;
  DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
  DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
  DMA_InitStructure.DMA_Channel = DMA_CHANNEL;    // added channel number
  DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Disable;  // added FIFO mode
  DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
  DMA_InitStructure.DMA_PeripheralBurst = DMA_MemoryBurst_Single;
  DMA_Init(DMA_STREAM, &DMA_InitStructure);

  SPI_StructInit(&SPI_InitStructure);
  SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex; //SPI_Direction_1Line_Tx;
  SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
  SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
  SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
  SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge;
  SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_8;
  SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
  SPI_InitStructure.SPI_CRCPolynomial = 7;
  SPI_Init(SPI1, &SPI_InitStructure);
  
  SPI_CalculateCRC(SPI1, DISABLE);
  
  nvic.NVIC_IRQChannel = DMA_STREAM_IRQ;
  nvic.NVIC_IRQChannelPreemptionPriority = 0;
  nvic.NVIC_IRQChannelSubPriority = 1; //0; ???
  nvic.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&nvic);

  DMA_STREAM->CR &= ~DMA_SxCR_EN;     // clear the EN bit to disable the stream
  DMA_STREAM->NDTR = VTOTAL;        // set number of bytes to transfer
  DMA_STREAM->M0AR = (u32) &fb[0][0];   // set start of frame buffer
  
  SPI_I2S_DMACmd(SPI1, SPI_I2S_DMAReq_Tx, ENABLE);  // allow Tx interrupt to generate DMA requests
  SPI_Cmd(SPI1, ENABLE);

  DMA_ITConfig(DMA_STREAM, DMA_IT_TC, ENABLE);
  DMA_STREAM->CR |= DMA_SxCR_EN;      // set the EN bit to enable the stream
}

//*****************************************************************************
//  This irq is generated at the end of the horizontal back porch.
//  Test if inside a valid vertical start frame (vflag variable), 
//  and start the DMA to output a single frame buffer line through the SPI device.
//*****************************************************************************
//__irq void TIM1_CC_IRQHandler(void)
__irq void TIM3_IRQHandler(void)
{
  if (vflag)
  {
///    DMA1_Channel3->CCR = 0x93;
    DMA_STREAM->CR |= DMA_SxCR_EN;    // set the EN bit to enable the stream
  }
  TIM3->SR &= ~TIM_IT_CC3;
}

//*****************************************************************************
//  This irq is generated at the end of the vertical back porch.
//  Sets the 'vflag' variable to 1 (valid vertical frame).
//*****************************************************************************
__irq void TIM4_IRQHandler(void)
{
  vflag = 1;
  TIM4->SR &= ~TIM_IT_CC3;
}

//*****************************************************************************
//  This interrupt is generated at the end of every line.
//  It will increment the line number and set the corresponding line pointer
//  in the DMA register.
//*****************************************************************************
///__irq void DMA1_Channel3_IRQHandler(void)
__irq void  DMA_STREAM_IRQHANDLER(void)
{ 
///  DMA1->IFCR = DMA1_IT_TC3;
///  DMA1_Channel3->CCR = 0x92;
///  DMA1_Channel3->CNDTR = VTOTAL;
  VIDEO_DMA->LIFCR = DMA_LIFCR_CTCIF3;  // clear the transfer complete interrupt flag
  DMA_STREAM->CR &= ~DMA_SxCR_EN;     // clear the EN bit to disable the stream
  DMA_STREAM->NDTR = VTOTAL;        // reload the number of bytes to send
  
  vdraw++;
  
  if (vdraw == 2)
  {
    vdraw = 0;

    vline++;
    
    if (vline == VID_VSIZE)
    {
      vdraw = vline = vflag = 0;
      DMA_STREAM->M0AR = (u32) &fb[0][0]; ///      DMA1_Channel3->CMAR = (u32) &fb[0][0];
      st_TIM4_IRQHandler++;
      if (!GPIO_ReadInputDataBit(GPIOB,GPIO_Pin_12))  Key_mode_cnt++; else {Key_mode_cnt=0; Key_mode=0; {if(Key_mode_single==1) Key_mode_single=2; else Key_mode_single=0;}}
      if (Key_mode_cnt>2) {Key_mode_cnt=2; Key_mode=1; if (Key_mode_single==0) Key_mode_single=1;}
      if (!GPIO_ReadInputDataBit(GPIOB,GPIO_Pin_14))  Key_plus_cnt++; else {Key_plus_cnt=0; Key_plus=0;}
      if (Key_plus_cnt>2) {Key_plus_cnt=2; Key_plus=1; Key_mode_single=3;}
      if (!GPIO_ReadInputDataBit(GPIOB,GPIO_Pin_13)) Key_minus_cnt++; else {Key_minus_cnt=0; Key_minus=0;}
      if (Key_minus_cnt>2) {Key_minus_cnt=2; Key_minus=1; Key_mode_single=3;}
    } else {
      DMA_STREAM->M0AR += VTOTAL; ///      DMA1_Channel3->CMAR += VTOTAL;
    }
  }
}

void vidClearScreen(void)
{
  u16 x, y;

  for (y = 0; y < VID_VSIZE; y++)
  {
    for (x = 0; x < VTOTAL; x++)
    {
      fb[y][x] = 0;
    }
  }
}

void vidInit(void)
{
  SPI_Configuration();
  TIMER_Configuration();
  vidClearScreen();
}
